@@ -2,7 +2,7 @@
 	decl.h
 		Type declarations
 		this file is part of Vegas
-		last modified 21 Dec 11 th
+		last modified 11 Apr 14 th
 */
 
 
@@ -29,15 +29,17 @@ typedef struct {
 typedef const Cumulants cCumulants;
 
 typedef int (*Integrand)(ccount *, creal *, ccount *, real *,
-  void *, creal *, cint *);
+  void *, cnumber *, creal *, cint *);
 
 typedef struct _this {
   count ndim, ncomp;
 #ifndef MLVERSION
   Integrand integrand;
   void *userdata;
+  number nvec;
+  subroutine initfun, exitfun;
 #ifdef HAVE_FORK
-  int ncores, *child;
+  int *child, ncores;
   SHM_ONLY(int shmid;)
 #endif
 #endif
